/*
**  ScriptManager.m
**
**  Copyright (c) 2003
**
**  Author: Yen-Ju Chen <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "ScriptManager.h"
#include <CodeEditorView/CodeEditorView.h>
#include <AppKit/AppKit.h>

static ScriptManager *sharedManager;

@implementation ScriptManager

+ (ScriptManager *) sharedScriptManager
{
  if (!sharedManager)
    {
      sharedManager = [[ScriptManager alloc] init];
    }
  return sharedManager;
}

- (CodeEditorView *) currentCodeEditorView
{
  return currentCEView;
}

- (void) setCodeEditorView: (CodeEditorView *) ceView
{
  ASSIGN(currentCEView, ceView);
}

- (void) dealloc
{
  RELEASE(currentCEView);
}

@end
