/*
**  CEView+Fontification.h
**
**  Copyright (c) 2002, 2003
**
**  Author: Yen-Ju Chen <yjchenx@hotmail.com>
**          Bjoern Giesler <bjoern@giesler.de>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _CodeEditorBundle_H_CEView_Fontification
#define _CodeEditorBundle_H_CEView_Fontification

#include "CodeEditorView.h"

@interface CodeEditorView (Fontification)

- (BOOL) supportLanguage: (NSString *) language;
- (void) fontifyAccordingToLanguage: (NSString *) language;
- (void) fontifyAccordingToLanguage: (NSString *) language
                            inRange: (NSRange) range;
@end

#endif /* _CodeEditorBundle_H_CEView_Fontification */

